@@ -138,6 +138,7 @@ const char * text_fuzzy_statuses[] = {
     "max min miscalculation",
     "A string for comparison was larger than the value of HUGE defined in the code.",
     "An attempt was made to use the maximum edit distance which was unset.",
+    "miscount",
 };
 
 #define STATIC static
@@ -226,6 +227,16 @@ typedef struct text_fuzzy_string {
 }
 text_fuzzy_string_t;
 
+/* Match candidates. */
+
+typedef struct candidate candidate_t;
+
+struct candidate {
+    int distance;
+    int offset;
+    candidate_t * next;
+};
+
 /* The following structure contains one string plus additional
    paraphenalia used in searching for the string, for example the
    alphabet of the string. */
@@ -274,6 +285,12 @@ typedef struct text_fuzzy {
     /* A character which is not in use. */
     unsigned char invalid_char;
 
+    /* Candidates for an array match. */
+
+    candidate_t first;
+    candidate_t * last;
+    int offset;
+
     /* Does the user want to use an alphabet filter? Default is yes,
        so this must be set to a non-zero value to switch off use. */
     unsigned int user_no_alphabet : 1;
@@ -301,6 +318,9 @@ typedef struct text_fuzzy {
 
     /* Are we scanning a list of entries? */
     unsigned int scanning : 1;
+
+    /* Do we want an array of answers? */
+    unsigned int wantarray : 1;
 }
 text_fuzzy_t;
 
@@ -657,10 +677,89 @@ FUNC (compare_single) (text_fuzzy_t * tf)
 	if (tf->scanning) {
 	    tf->max_distance = tf->distance;
 	}
+	if (tf->wantarray) {
+	    candidate_t * c;
+	    c = malloc (sizeof (candidate_t));
+	    FAIL (! c, memory_error);
+	    tf->n_mallocs+=1;
+	    c->distance = d;
+	    c->offset = tf->offset;
+	    c->next = 0;
+	    tf->last->next = c;
+	    tf->last = c;
+	}
     }
     OK;
 }
 
+FUNC (get_candidates) (text_fuzzy_t * text_fuzzy,
+		       int * n_candidates_ptr,
+		       int ** candidates_ptr)
+{
+    candidate_t * c;
+    candidate_t * last;
+    int n_candidates = 0;
+    int * candidates;
+    int i;
+
+    last = text_fuzzy->first.next;
+    while (last) {
+	c = last;
+	last = last->next;
+	if (c->distance == text_fuzzy->distance) {
+	    n_candidates++;
+	}
+    }
+
+    if (n_candidates == 0) {
+	* n_candidates_ptr = 0;
+	* candidates_ptr = 0;
+	OK;
+    }
+
+    candidates = malloc (sizeof (int) * n_candidates);
+    FAIL (! candidates, memory_error);
+    text_fuzzy->n_mallocs+=1;
+
+    last = text_fuzzy->first.next;
+    i = 0;
+    while (last) {
+	c = last;
+
+	/* Set "last" to the next one here so that we do not
+	   access freed memory. */
+	last = last->next;
+	
+	/* Some of the entries might be things which had a lower
+	   distance initially, but then were beaten by later
+	   entries, so here we check that the entry actually does
+	   have the lowest distance, and only if so do we keep
+	   it. */
+	
+	if (c->distance == text_fuzzy->distance) {
+	    candidates[i] = c->offset;
+	    i++;
+	}
+	free (c);
+	text_fuzzy->n_mallocs--;
+    }
+    FAIL_MSG (i != n_candidates, miscount,
+	      "Wrong number of entries %d should be %d", i, n_candidates);
+    * candidates_ptr = candidates;
+    * n_candidates_ptr = n_candidates;
+    OK;
+}
+
+FUNC (free_candidates) (text_fuzzy_t * text_fuzzy, int * candidates)
+{
+    if (candidates) {
+	free (candidates);
+	text_fuzzy->n_mallocs--;
+    }
+    OK;
+}
+
+
 /* This is the threshold above which we do not bother computing the
    alphabet of the string. If it has more than this number of unique
    characters, the alphabet will not reduce the search time by
@@ -928,5 +1027,7 @@ status: max_distance_misuse
 An attempt was made to use the maximum edit distance which was unset.
 %%
 
+status: miscount
+
 */
 
